window.Gleam.initSearch([{"doc":"spoke","title":"spoke","content":"# spoke\n\n[![Package Version](https://img.shields.io/hexpm/v/spoke)](https://hex.pm/packages/spoke)\n[![Hex Docs](https://img.shields.io/badge/hex-docs-ffaff3)](https://hexdocs.pm/spoke/)\n\nSpoke is a MQTT 3.1.1 client written in Gleam for the Erlang runtime.\n\nExample usage:\n\n```sh\ngleam add spoke\ngleam add spoke_tcp\n```\n\n```gleam\nimport gleam/erlang/process\nimport gleam/int\nimport gleam/io\nimport gleam/string\nimport spoke\nimport spoke/tcp\n\npub fn main() {\n  let client_id = &quot;spoke&quot; &lt;&gt; string.inspect(int.random(999_999_999))\n  let topic = &quot;spoke-test&quot;\n\n  let client =\n    tcp.connector_with_defaults(&quot;test.mosquitto.org&quot;)\n    |&gt; spoke.connect_with_id(client_id)\n    |&gt; spoke.start_session\n\n  spoke.connect(client, True)\n\n  let assert Ok(_) =\n    spoke.subscribe(client, [spoke.SubscribeRequest(topic, spoke.AtLeastOnce)])\n\n  let message =\n    spoke.PublishData(\n      topic,\n      &lt;&lt;&quot;Hello from spoke!&quot;&gt;&gt;,\n      spoke.AtLeastOnce,\n      retain: False,\n    )\n  spoke.publish(client, message)\n\n  let updates = spoke.updates(client)\n\n  let assert Ok(spoke.ConnectionStateChanged(spoke.ConnectAccepted(_))) =\n    process.receive(updates, 1000)\n\n  let message = process.receive(updates, 1000)\n  io.println(string.inspect(message))\n\n  spoke.disconnect(client)\n}\n```\n\nThis should print the following,\nassuming `test.mosquitto.org` is up (it&#39;s not rare for it to be down):\n```\nOk(ReceivedMessage(&quot;spoke-test&quot;, &quot;Hello from spoke!&quot;, False))\n```\n\n## Design choices\n\nSpoke aspires to be as high-level as possible, without being opinionated.\nThis means that the only things you&#39;ll need to handle yourself are\npersistent session management and re-connections.\nIf you don&#39;t care about persistent sessions or overloading the server,\nsimply cleaning the session on each connect and\nimmediately reconnecting on unexpected disconnects should give you a reliable client.\n\n## Transport channels\n\nThe core `spoke` package is transport channel agnostic.\nAt the time of writing, `spoke_tcp` is the only implementation of transport channels for spoke.\nInstead of using `spoke_tcp`, you can also bring your own transport channel.\nSee `TransportChannelConnector` for the required functions to make this work.\n\nIf you&#39;d have use for a WebSocket channel,\nplease leave a feature request, and I might implement it.\n","url":"index.html"},{"doc":"spoke","title":"AuthDetails","content":"pub type AuthDetails {\n  AuthDetails(username: String, password: Option(BitArray))\n}\n Authentication details passed to the server when connecting.\n Remember that these are not encrypted,\n unless working with an encrypted transport channel.\nAuthDetails(username: String, password: Option(BitArray))\n\n\nSynonyms:\nspoke.AuthDetails\nspoke AuthDetails","url":"spoke.html#AuthDetails"},{"doc":"spoke","title":"Client","content":"pub opaque type Client\n An opaque handle for the MQTT client.\n Can live across multiple sessions and connections,\n or resume sessions that were previously disconnected.\n Under the hood, the client is an OTP actor.\n\nSynonyms:\nspoke.Client\nspoke Client","url":"spoke.html#Client"},{"doc":"spoke","title":"ConnectError","content":"pub type ConnectError {\n  UnacceptableProtocolVersion\n  IdentifierRefused\n  ServerUnavailable\n  BadUsernameOrPassword\n  NotAuthorized\n}\n Error code from the server -\n we got a response, but there was an error.\nUnacceptableProtocolVersion\n The MQTT server doesn&#39;t support MQTT 3.1.1\n\nIdentifierRefused\n The Client identifier is correct UTF-8 but not allowed by the Server\n\nServerUnavailable\n The Network Connection has been made but the MQTT service is unavailable\n\nBadUsernameOrPassword\n The data in the user name or password is malformed\n\nNotAuthorized\n The Client is not authorized to connect\n\nSynonyms:\nspoke.ConnectError\nspoke ConnectError","url":"spoke.html#ConnectError"},{"doc":"spoke","title":"ConnectOptions","content":"pub type ConnectOptions {\n  ConnectOptions(\n    connector: TransportChannelConnector,\n    client_id: String,\n    authentication: Option(AuthDetails),\n    keep_alive_seconds: Int,\n    server_timeout_ms: Int,\n  )\n}\n The set of options used to establish a connection to the server.\n Includes the set of data that should generally not change across\n multiple connect calls (if they are needed).\nConnectOptions(\n  connector: TransportChannelConnector,\n  client_id: String,\n  authentication: Option(AuthDetails),\n  keep_alive_seconds: Int,\n  server_timeout_ms: Int,\n)\n\nconnector\n&lt;p&gt;Function to establish a transport channel.&lt;/p&gt;\n\nclient_id\n&lt;p&gt;The MQTT client ID used when connecting to the server.&lt;/p&gt;\n\nauthentication\n&lt;p&gt;Optional username and (additionally optional) password&lt;/p&gt;\n\nkeep_alive_seconds\n&lt;p&gt;Keep-alive interval in seconds (MQTT spec doesn’t allow more granular control)&lt;/p&gt;\n\nserver_timeout_ms\n&lt;p&gt;“Reasonable amount of time” for the server to respond (including network latency),\nas used in the MQTT specification.&lt;/p&gt;\n\nSynonyms:\nspoke.ConnectOptions\nspoke ConnectOptions","url":"spoke.html#ConnectOptions"},{"doc":"spoke","title":"ConnectionState","content":"pub type ConnectionState {\n  ConnectFailed(String)\n  ConnectRejected(ConnectError)\n  ConnectAccepted(session_present: Bool)\n  Disconnected\n  DisconnectedUnexpectedly(reason: String)\n}\n Represents the state of the connection to the server.\nConnectFailed(String)\n Connecting to the server failed before we got a response\n to the connect packet.\n\nConnectRejected(ConnectError)\n The server was reachable, but rejected our connect packet\n\nConnectAccepted(session_present: Bool)\n The server has accepted our connect packet\n\nDisconnected\n Disconnected as a result of calling `disconnect`\n\nDisconnectedUnexpectedly(reason: String)\n The connection was dropped for an unexpected reason,\n e.g. a transport channel error or protocol violation.\n\nSynonyms:\nspoke.ConnectionState\nspoke ConnectionState","url":"spoke.html#ConnectionState"},{"doc":"spoke","title":"OperationError","content":"pub type OperationError {\n  NotConnected\n  OperationTimedOut\n  ProtocolViolation\n  KilledDuringOperation\n}\n Unified error type for operations that are completed in a blocking way.\nNotConnected\n The client was not connected when it was required.\n\nOperationTimedOut\n The operation did not complete in time.\n\nProtocolViolation\n We received unexpected data from the server, and will disconnect.\n\nKilledDuringOperation\n The client actor was killed while the operation was in progress.\n\nSynonyms:\nspoke.OperationError\nspoke OperationError","url":"spoke.html#OperationError"},{"doc":"spoke","title":"PublishData","content":"pub type PublishData {\n  PublishData(\n    topic: String,\n    payload: BitArray,\n    qos: QoS,\n    retain: Bool,\n  )\n}\n A convenience record to hold all the data used when publishing messages.\nPublishData(\n  topic: String,\n  payload: BitArray,\n  qos: QoS,\n  retain: Bool,\n)\n\n\nSynonyms:\nspoke.PublishData\nspoke PublishData","url":"spoke.html#PublishData"},{"doc":"spoke","title":"QoS","content":"pub type QoS {\n  AtMostOnce\n  AtLeastOnce\n  ExactlyOnce\n}\n Quality of Service levels, as specified in the MQTT specification\nAtMostOnce\n The message is delivered according to the capabilities of the underlying network.\n No response is sent by the receiver and no retry is performed by the sender.\n The message arrives at the receiver either once or not at all.\n\nAtLeastOnce\n This quality of service ensures that the message arrives at the receiver at least once.\n\nExactlyOnce\n This is the highest quality of service,\n for use when neither loss nor duplication of messages are acceptable.\n There is an increased overhead associated with this quality of service.\n\nSynonyms:\nspoke.QoS\nspoke QoS","url":"spoke.html#QoS"},{"doc":"spoke","title":"SubscribeRequest","content":"pub type SubscribeRequest {\n  SubscribeRequest(filter: String, qos: QoS)\n}\n Utility record for the data required to request a subscription.\nSubscribeRequest(filter: String, qos: QoS)\n\n\nSynonyms:\nspoke.SubscribeRequest\nspoke SubscribeRequest","url":"spoke.html#SubscribeRequest"},{"doc":"spoke","title":"Subscription","content":"pub type Subscription {\n  SuccessfulSubscription(topic_filter: String, qos: QoS)\n  FailedSubscription(topic_filter: String)\n}\n The result of a subscribe operation\nSuccessfulSubscription(topic_filter: String, qos: QoS)\n The subscribe succeeded with the specified QoS level.\n\nFailedSubscription(topic_filter: String)\n The server returned a failure for requested subscription.\n\nSynonyms:\nspoke.Subscription\nspoke Subscription","url":"spoke.html#Subscription"},{"doc":"spoke","title":"TransportChannelConnector","content":"pub type TransportChannelConnector =\n  fn() -&gt;\n    Result(\n      #(\n        fn(BytesTree) -&gt; Result(Nil, String),\n        fn() -&gt; Selector(Result(BitArray, String)),\n        fn() -&gt; Nil,\n      ),\n      String,\n    )\n A function that connects a transport channel and\n returns the send, receive, and shutdown functions or an error.\n NOTE: This only uses standard library and Erlang types,\n so that implementors do not need to depend on the Spoke package.\n\nSynonyms:\nspoke.TransportChannelConnector\nspoke TransportChannelConnector","url":"spoke.html#TransportChannelConnector"},{"doc":"spoke","title":"Update","content":"pub type Update {\n  ReceivedMessage(\n    topic: String,\n    payload: BitArray,\n    retained: Bool,\n  )\n  ConnectionStateChanged(ConnectionState)\n}\n Represents a received message or change in the client.\nReceivedMessage(topic: String, payload: BitArray, retained: Bool)\n A published message to a topic this client was subscribed to was received.\n\nConnectionStateChanged(ConnectionState)\n The connection state of this client changed.\n\nSynonyms:\nspoke.Update\nspoke Update","url":"spoke.html#Update"},{"doc":"spoke","title":"connect","content":"pub fn connect(client: Client, clean_session: Bool) -&gt; Nil\n Starts connecting to the MQTT server.\n The connection state will be published as an update.\n If a connection is already established or being established,\n this will be a no-op.\n Note that switching between `clean_session` values\n while already connecting is currently not well handled.\nSynonyms:\nspoke.connect\nspoke connect","url":"spoke.html#connect"},{"doc":"spoke","title":"connect_with_id","content":"pub fn connect_with_id(\n  connector: fn() -&gt;\n    Result(\n      #(\n        fn(BytesTree) -&gt; Result(Nil, String),\n        fn() -&gt; Selector(Result(BitArray, String)),\n        fn() -&gt; Nil,\n      ),\n      String,\n    ),\n  client_id: String,\n) -&gt; ConnectOptions\n Constructs connect options from transport options, the given client id,\n and default settings for the rest of the options.\nSynonyms:\nspoke.connect_with_id\nspoke connect_with_id","url":"spoke.html#connect_with_id"},{"doc":"spoke","title":"connect_with_will","content":"pub fn connect_with_will(\n  client: Client,\n  clean_session: Bool,\n  will: PublishData,\n) -&gt; Nil\n Same as `connect`, but specifies a will message for this connection.\nSynonyms:\nspoke.connect_with_will\nspoke connect_with_will","url":"spoke.html#connect_with_will"},{"doc":"spoke","title":"disconnect","content":"pub fn disconnect(client: Client) -&gt; String\n Disconnects from the MQTT server.\n The connection state change will also be published as an update.\n If a connection is not established or being established,\n this will be a no-op.\n Returns the serialized session state to be potentially restored later.\nSynonyms:\nspoke.disconnect\nspoke disconnect","url":"spoke.html#disconnect"},{"doc":"spoke","title":"keep_alive_seconds","content":"pub fn keep_alive_seconds(\n  options: ConnectOptions,\n  keep_alive_seconds: Int,\n) -&gt; ConnectOptions\n Builder function for specifying the keep-alive time in the connect options.\nSynonyms:\nspoke.keep_alive_seconds\nspoke keep_alive_seconds","url":"spoke.html#keep_alive_seconds"},{"doc":"spoke","title":"pending_publishes","content":"pub fn pending_publishes(client: Client) -&gt; Int\n Returns the number of QoS &gt; 0 publishes that haven&#39;t yet been completely published.\n Also see `wait_for_publishes_to_finish`.\nSynonyms:\nspoke.pending_publishes\nspoke pending_publishes","url":"spoke.html#pending_publishes"},{"doc":"spoke","title":"publish","content":"pub fn publish(client: Client, data: PublishData) -&gt; Nil\n Connects to the MQTT server.\n Will disconnect if the connect times out,\n and send the `Disconnect` update.\n Note that in case of a timeout,\n the message might still have been already published.\nSynonyms:\nspoke.publish\nspoke publish","url":"spoke.html#publish"},{"doc":"spoke","title":"restore_session","content":"pub fn restore_session(\n  connect_options: ConnectOptions,\n  state: String,\n) -&gt; Result(Client, String)\n Restores an MQTT session from a previously stored state,\n as returned by `disconnect`.\n Will return an error if the session state can not be restored\n from the provided string.\n Does not connect to the server, until `connect` is called.\nSynonyms:\nspoke.restore_session\nspoke restore_session","url":"spoke.html#restore_session"},{"doc":"spoke","title":"server_timeout_ms","content":"pub fn server_timeout_ms(\n  options: ConnectOptions,\n  server_timeout_ms: Int,\n) -&gt; ConnectOptions\n Builder function for specifying the server operation timeout in the connect options.\nSynonyms:\nspoke.server_timeout_ms\nspoke server_timeout_ms","url":"spoke.html#server_timeout_ms"},{"doc":"spoke","title":"start_session","content":"pub fn start_session(connect_options: ConnectOptions) -&gt; Client\n Starts a new MQTT session with the given options.\n Does not connect to the server, until `connect` is called.\nSynonyms:\nspoke.start_session\nspoke start_session","url":"spoke.html#start_session"},{"doc":"spoke","title":"subscribe","content":"pub fn subscribe(\n  client: Client,\n  topics: List(SubscribeRequest),\n) -&gt; Result(List(Subscription), OperationError)\n Subscribes to the given topics.\n Will block until we get a response from the server,\n returning the result of the operation.\nSynonyms:\nspoke.subscribe\nspoke subscribe","url":"spoke.html#subscribe"},{"doc":"spoke","title":"unsubscribe","content":"pub fn unsubscribe(\n  client: Client,\n  topics: List(String),\n) -&gt; Result(Nil, OperationError)\n Unsubscribes from the given topics.\n Will block until we get a response from the server,\n returning the result of the operation.\nSynonyms:\nspoke.unsubscribe\nspoke unsubscribe","url":"spoke.html#unsubscribe"},{"doc":"spoke","title":"updates","content":"pub fn updates(client: Client) -&gt; Subject(Update)\n Returns a `Subject` for receiving client updates\n (received messages and connection state changes).\n NOTE: This function is provided for convenience\n as receiving updates only works from the process that created the client!\n Future versions of spoke might support receiving updates in multiple processes. \nSynonyms:\nspoke.updates\nspoke updates","url":"spoke.html#updates"},{"doc":"spoke","title":"using_auth","content":"pub fn using_auth(\n  options: ConnectOptions,\n  username: String,\n  password: Option(BitArray),\n) -&gt; ConnectOptions\n Builder function for specifying the authentication details to be used when connecting.\nSynonyms:\nspoke.using_auth\nspoke using_auth","url":"spoke.html#using_auth"},{"doc":"spoke","title":"wait_for_publishes_to_finish","content":"pub fn wait_for_publishes_to_finish(\n  client: Client,\n  timeout: Int,\n) -&gt; Result(Nil, Nil)\n Wait for all pending QoS &gt; 0 publishes to complete.\n Returns an error if the operation times out,\n or client is killed while waiting.\nSynonyms:\nspoke.wait_for_publishes_to_finish\nspoke wait_for_publishes_to_finish","url":"spoke.html#wait_for_publishes_to_finish"},{"doc":"spoke","title":"spoke","content":"","url":"spoke.html"}]);